/** 
 ** This application achieves the single operation of producing a 
 ** an JPEG file whose pixel values are the color value average
 ** of the images appearing in the given directory.
 **  
 ** Usage: java JPGavg dirname avg.jpg
 **
 ** Author: L. Sweeney, For: Privacy and Anonymity in Data, Date: 10/2003.
 **/

import java.lang.reflect.Array;
import java.awt.*; 
import java.awt.event.*; 
import java.awt.image.*; 
import java.awt.geom.GeneralPath; 
import com.sun.image.codec.jpeg.*; 
import java.io.*; 
import javax.imageio.*; 

public class JPGavg {

   /* Contains the pixel value sums of JPEG files. */
   public static long [] datasum;

   public static int width;
   public static int height;
   public static int total = 0;


   /* This method returns an int containing a concatenated 24-bit value
    * of three 8-bit bytes, b followed by g followed by r.  Each of the
    * values b, g, and r are 8-bit values (0 to 255).
    */
   public static int getColor(int b, int g, int r) {
      return (int)Math.pow(2,16)*r + 256*g + b;
   }

   public static void initSumArray(int w, int h) {
      width = w;
      height = h;
      datasum = new long[width*height*3];
      for(int i=0; i < width*height*3; i++) {
         datasum[i] = 0;
      }
   }


   public static void main(String [] args) {
      try {

         File dirname = new File(args[0]);
         File [] files = dirname.listFiles();
         File f;
         String filename;
         BufferedImage bufi;
         Raster data;
         DataBuffer datab;
         for(int i = 0; i < Array.getLength(files); i++) {

            //Read JPEG image from an input file 
            filename = dirname + "/" + files[i].getName();
            if( filename.endsWith(".jpg") ) {
               f = new File(filename); 
               bufi = ImageIO.read(f);  
               System.out.print("Loaded JPEG file: " + filename);
               data = bufi.getData();
               datab = data.getDataBuffer();
               if (total == 0) initSumArray(data.getWidth(), data.getHeight());

               //Add pixel values to sum array, datasum, if same size.
               if((data.getWidth() == width) && (data.getHeight() == height)) {
                  for(int j=0; j < datab.getSize(); j++) 
                     datasum[j] += datab.getElem(j);
                  System.out.println(". Added.");
                  total++;
               } else {
                  System.out.println(". Skip. Different dimensions.");
               }
            } else {
                  System.out.println("Skip " + filename + ". Not end in .jpg.");
            }
          }

          // Convert 3 color bytes to a single 24-bit BGR pixel value.
          int [] datanew = new int[width*height];
          for(int pixpos=0, p=0; pixpos < width*height; pixpos++, p+=3)
             datanew[pixpos] 
                = getColor((int)(datasum[p]/total), 
                      (int)(datasum[p+1]/total), (int)(datasum[p+2]/total));
          BufferedImage bufnew 
             = new BufferedImage(width, height, BufferedImage.TYPE_3BYTE_BGR); 
          bufnew.setRGB(0, 0, width, height, datanew, 0, width); 

          // Save resulting image to file.
          System.out.print("Writing pixel vector to file...");
          File fnew = new File(args[1]);
          ImageIO.write(bufnew, "jpg", fnew);
          System.out.println("done.");

      } catch (Exception e) { 
            System.out.println("\nProblem encountered!");
            System.out.println("Usage: dirname avg.jpg");
            System.out.println("Creates avg.jpg as the pixel-wise average of the .jpg files in dirname."); 
      }
   }


}